/*
 * Author:  Konrad Rzeszutek <conradus@mindless.com>
 * Date:    Oct 19th 1999
 * File:    Editor3D/FaceObject.java
 *
 * Notes:
 *
 *   A face is set of various 3D/2D points.
 *   This object holds the point(s) assoicated with this face. It also
 *   has some basic method to matrix multiply all the points against a 
 *   matrix.
 *   
 */
 package Editor3D;
 
import java.util.*;

public class FaceObject {

    /*
     * Initalizes the FaceObject.
     *
     * require:
     *   size >=- 1
     *
     */    
    public FaceObject(int size) {
        
	vec = new Vector(size,0);
	capacity = size;
    }
    
    /*
     * Adds a point to the face.
     *
     * require:
     *   elmnt !- null && (this.size() < this.capacity)
     *
     */
    public void add(VectorObject elmnt) {
    
	vec.addElement( (Object) elmnt);
    }
    
    /*
     * Receives the specified VectorObject.
     *
     * require:
     *  0 <= index <= capacity.
     *
     */
    public VectorObject get(int index) {
    
	return (VectorObject) vec.elementAt(index);
    }
    
    /*
     * Returns the amount of points currently deposited.
     *
     * return:
     *   >= 0
     *
     */
    public int size() {
    
	return vec.size();
    }
    
    /*
     * Multiply all points held by this face against m matrix.
     *
     * require:
     *  m.rows == m.columns == 4
     *
     */
    public void multiply(Matrix m) {
    
	// Dot product all points against m Matrix,
	       System.out.print("["+vec.size()+"]");
    	for (int i = 0; i < vec.size(); i++) {
	    ((VectorObject) (vec.elementAt(i))).multiply(m);
	    //System.out.println(vec.elementAt(i));
	    }

    }
    /*
     * Multiply the WC of all the points held by the face against the m matrix.
     *
     * require:
     *  m.rows == m.columns == 4.
     *
     */
    public void  recalcWC(Matrix m) {
    
    	for (int i = 0; i < vec.size(); i++) 
	    ((VectorObject) (vec.elementAt(i))).recalcWC(m);

    }

    public void homogonize() {
        
    	for (int i = 0; i < vec.size(); i++) 
	    ((VectorObject) (vec.elementAt(i))).homogonize();

    }

    
    /*
     * Give the normal of this face. This method does:
     *  - finds the centroid.
     *  - cross product one verticie with the centroid (this is the normal)
     *  - return the normal
     *
     * return:
     *   VectorObject (must be in WC)
     */
    public boolean backFace(VectorObject vecCOP, VectorObject vecLight2) {
	
	VectorObject cen = new VectorObject();
	
	// In WC
	for (int i =0; i < vec.size(); i++) 
	    cen.addVectors(((VectorObject) (vec.elementAt(i))));
	
	cen.scalarMultiply( 1.0 / vec.size());

	VectorObject v1 = new VectorObject();
	VectorObject v2 = new VectorObject();
	
	v1.subtractVectors( ((VectorObject) (vec.elementAt(0))), ((VectorObject) (vec.elementAt(1))));
	v2.subtractVectors(  ((VectorObject) (vec.elementAt(2))),  ((VectorObject) (vec.elementAt(0))));
	
	// Normal in VRC
	VectorObject vecNormal = VectorObject.unitVector2(VectorObject.crossProduct2(v2, v1));
	// Normal in WC
	VectorObject vecNormalWC = VectorObject.unitVector(VectorObject.crossProduct(v2, v1));
	// COP in VRC
	VectorObject vecCop = VectorObject.unitVector2(vecCOP);
	
	backface = false;
	// Backface calculated in VRC
	backface = (VectorObject.dotProduct2(vecCop, vecNormal) > 0.0);
	
	if (backface) {	   // Do something usefulll
	    // Calculate the intentisty.
	    VectorObject vecLight= new VectorObject();
	    vecLight.subtractVectors(cen, vecLight2);
	    vecLight = VectorObject.unitVector(vecLight);
	    illum = (int) (( VectorObject.dotProduct(vecLight, vecNormalWC))*5 + 5);
	}
	
	return backface;
    }
   /*
    * Reset all the data to the original given ones.
    *
    */
    public void reset() {

       	for (int i = 0; i < vec.size(); i++)
	    ((VectorObject) (vec.elementAt(i))).reset();
   }
   /*
    * String representation off the face (including the points).
    *
    * return:
    *   String.
    *
    */
    public String toString() {
    
	String s = "Points in this face:\n";
	for (int i = 0; i < vec.size(); i++)
	    s+= vec.elementAt(i).toString();
	return s;
    }

    boolean backface;
    private Vector vec; //List of all points.
    int capacity;
    int illum;
}
